@@ -2,7 +2,7 @@
  * commit.c:  wrappers around wc commit functionality.
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -23,7 +23,6 @@
 /*** Includes. ***/
 
 #include <string.h>
-#include <assert.h>
 #include <apr_strings.h>
 #include <apr_hash.h>
 #include <apr_md5.h>
@@ -36,9 +35,9 @@
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_error_codes.h"
+#include "svn_dirent_uri.h"
 #include "svn_path.h"
 #include "svn_io.h"
-#include "svn_md5.h"
 #include "svn_time.h"
 #include "svn_sorts.h"
 #include "svn_props.h"
@@ -75,6 +74,9 @@ typedef struct import_ctx_t
 
    Fill DIGEST with the md5 checksum of the sent file; DIGEST must be
    at least APR_MD5_DIGESTSIZE bytes long. */
+
+/* ### how does this compare against svn_wc_transmit_text_deltas2() ??? */
+
 static svn_error_t *
 send_file_contents(const char *path,
                    void *file_baton,
@@ -83,11 +85,9 @@ send_file_contents(const char *path,
                    unsigned char *digest,
                    apr_pool_t *pool)
 {
-  const char *tmpfile_path = NULL;
   svn_stream_t *contents;
   svn_txdelta_window_handler_t handler;
   void *handler_baton;
-  apr_file_t *f;
   const svn_string_t *eol_style_val = NULL, *keywords_val = NULL;
   svn_boolean_t special = FALSE;
   svn_subst_eol_style_t eol_style;
@@ -124,39 +124,41 @@ send_file_contents(const char *path,
   else
     keywords = NULL;
 
-  /* If we have EOL styles or keywords to de-translate, do it.  */
-  if (svn_subst_translation_required(eol_style, eol, keywords, special, TRUE))
+  if (special)
     {
-      const char *temp_dir;
+      SVN_ERR(svn_subst_read_specialfile(&contents, path, pool, pool));
+    }
+  else
+    {
+      /* Open the working copy file. */
+      SVN_ERR(svn_stream_open_readonly(&contents, path, pool, pool));
 
-      /* Now create a new tempfile, and open a stream to it. */
-      SVN_ERR(svn_io_temp_dir(&temp_dir, pool));
-      SVN_ERR(svn_io_open_unique_file2
-              (NULL, &tmpfile_path,
-               svn_path_join(temp_dir, "svn-import", pool),
-               ".tmp", svn_io_file_del_on_pool_cleanup, pool));
+      /* If we have EOL styles or keywords, then detranslate the file. */
+      if (svn_subst_translation_required(eol_style, eol, keywords,
+                                         FALSE, TRUE))
+        {
+          svn_boolean_t repair = FALSE;
 
-      SVN_ERR(svn_subst_translate_to_normal_form
-              (path, tmpfile_path, eol_style, eol, FALSE,
-               keywords, special, pool));
+          if (eol_style == svn_subst_eol_style_native)
+            eol = SVN_SUBST_NATIVE_EOL_STR;
+          else if (eol_style == svn_subst_eol_style_fixed)
+            repair = TRUE;
+          else if (eol_style != svn_subst_eol_style_none)
+            return svn_error_create(SVN_ERR_IO_UNKNOWN_EOL, NULL, NULL);
+
+          /* Wrap the working copy stream with a filter to detranslate it. */
+          contents = svn_subst_stream_translated(contents,
+                                                 eol,
+                                                 repair,
+                                                 keywords,
+                                                 FALSE /* expand */,
+                                                 pool);
+        }
     }
 
-  /* Open our contents file, either the original path or the temporary
-     copy we might have made above. */
-  SVN_ERR(svn_io_file_open(&f, tmpfile_path ? tmpfile_path : path,
-                           APR_READ, APR_OS_DEFAULT, pool));
-  contents = svn_stream_from_aprfile(f, pool);
-
   /* Send the file's contents to the delta-window handler. */
-  SVN_ERR(svn_txdelta_send_stream(contents, handler, handler_baton,
-                                  digest, pool));
-
-  /* Close our contents file. */
-  SVN_ERR(svn_io_file_close(f, pool));
-
-  /* The temp file is removed by the pool cleanup run by the caller */
-
-  return SVN_NO_ERROR;
+  return svn_txdelta_send_stream(contents, handler, handler_baton,
+                                 digest, pool);
 }
 
 
@@ -252,10 +254,11 @@ import_file(const svn_delta_editor_t *editor,
                              properties, digest, pool));
 
   /* Finally, close the file. */
-  text_checksum = svn_md5_digest_to_cstring(digest, pool);
-  SVN_ERR(editor->close_file(file_baton, text_checksum, pool));
+  text_checksum =
+    svn_checksum_to_cstring(svn_checksum__from_digest(digest, svn_checksum_md5,
+                                                      pool), pool);
 
-  return SVN_NO_ERROR;
+  return editor->close_file(file_baton, text_checksum, pool);
 }
 
 
@@ -586,17 +589,14 @@ import(const char *path,
     }
 
   if (import_ctx->repos_changed)
-    SVN_ERR(editor->close_edit(edit_baton, pool));
+    return editor->close_edit(edit_baton, pool);
   else
-    SVN_ERR(editor->abort_edit(edit_baton, pool));
-
-  return SVN_NO_ERROR;
+    return editor->abort_edit(edit_baton, pool);
 }
 
 
 static svn_error_t *
 get_ra_editor(svn_ra_session_t **ra_session,
-              svn_revnum_t *latest_rev,
               const svn_delta_editor_t **editor,
               void **edit_baton,
               svn_client_ctx_t *ctx,
@@ -636,10 +636,6 @@ get_ra_editor(svn_ra_session_t **ra_session,
                                  base_url);
     }
 
-  /* Fetch the latest revision if requested. */
-  if (latest_rev)
-    SVN_ERR(svn_ra_get_latest_revnum(*ra_session, latest_rev, pool));
-
   SVN_ERR(svn_client__ensure_revprop_table(&commit_revprops, revprop_table,
                                            log_msg, ctx, pool));
 
@@ -691,8 +687,7 @@ svn_client_import3(svn_commit_info_t **commit_info_p,
       apr_array_header_t *commit_items
         = apr_array_make(pool, 1, sizeof(item));
 
-      SVN_ERR(svn_client_commit_item_create
-              ((const svn_client_commit_item3_t **) &item, pool));
+      item = svn_client_commit_item3_create(pool);
       item->path = apr_pstrdup(pool, path);
       item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
       APR_ARRAY_PUSH(commit_items, svn_client_commit_item3_t *) = item;
@@ -741,9 +736,9 @@ svn_client_import3(svn_commit_info_t **commit_info_p,
           url = temp;
         }
     }
-  while ((err = get_ra_editor(&ra_session, NULL,
+  while ((err = get_ra_editor(&ra_session,
                               &editor, &edit_baton, ctx, url, base_dir,
-                              NULL, log_msg, NULL, revprop_table, 
+                              NULL, log_msg, NULL, revprop_table,
                               commit_info_p, FALSE, NULL, TRUE, subpool)));
 
   /* Reverse the order of the components we added to our NEW_ENTRIES array. */
@@ -801,20 +796,7 @@ svn_client_import3(svn_commit_info_t **commit_info_p,
 
   /* Transfer *COMMIT_INFO from the subpool to the callers pool */
   if (*commit_info_p)
-    {
-      svn_commit_info_t *tmp_commit_info;
-
-      tmp_commit_info = svn_create_commit_info(pool);
-      *tmp_commit_info = **commit_info_p;
-      if (tmp_commit_info->date)
-        tmp_commit_info->date = apr_pstrdup(pool, tmp_commit_info->date);
-      if (tmp_commit_info->author)
-        tmp_commit_info->author = apr_pstrdup(pool, tmp_commit_info->author);
-      if (tmp_commit_info->post_commit_err)
-        tmp_commit_info->post_commit_err
-          = apr_pstrdup(pool, tmp_commit_info->post_commit_err);
-      *commit_info_p = tmp_commit_info;
-    }
+    *commit_info_p = svn_commit_info_dup(*commit_info_p, pool);
 
   svn_pool_destroy(subpool);
 
@@ -822,40 +804,6 @@ svn_client_import3(svn_commit_info_t **commit_info_p,
 }
 
 
-svn_error_t *
-svn_client_import2(svn_commit_info_t **commit_info_p,
-                   const char *path,
-                   const char *url,
-                   svn_boolean_t nonrecursive,
-                   svn_boolean_t no_ignore,
-                   svn_client_ctx_t *ctx,
-                   apr_pool_t *pool)
-{
-  return svn_client_import3(commit_info_p,
-                            path, url,
-                            SVN_DEPTH_INFINITY_OR_FILES(! nonrecursive),
-                            no_ignore, FALSE, NULL, ctx, pool);
-}
-
-svn_error_t *
-svn_client_import(svn_client_commit_info_t **commit_info_p,
-                  const char *path,
-                  const char *url,
-                  svn_boolean_t nonrecursive,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
-{
-  svn_commit_info_t *commit_info = NULL;
-  svn_error_t *err;
-
-  err = svn_client_import2(&commit_info,
-                           path, url, nonrecursive,
-                           FALSE, ctx, pool);
-  /* These structs have the same layout for the common fields. */
-  *commit_info_p = (svn_client_commit_info_t *) commit_info;
-  return err;
-}
-
 static svn_error_t *
 remove_tmpfiles(apr_hash_t *tempfiles,
                 apr_pool_t *pool)
@@ -1191,7 +1139,7 @@ struct post_commit_baton
   svn_wc_adm_access_t *base_dir_access;
   svn_boolean_t keep_changelists;
   svn_boolean_t keep_locks;
-  apr_hash_t *digests;
+  apr_hash_t *checksums;
 };
 
 static svn_error_t *
@@ -1243,15 +1191,13 @@ post_process_commit_item(void *baton, void *this_item, apr_pool_t *pool)
 
   /* Allocate the queue in a longer-lived pool than (iter)pool:
      we want it to survive the next iteration. */
-  SVN_ERR(svn_wc_queue_committed
-          (&(btn->queue),
-           item->path, adm_access, loop_recurse,
-           item->incoming_prop_changes,
-           remove_lock, (! btn->keep_changelists),
-           apr_hash_get(btn->digests, item->path, APR_HASH_KEY_STRING),
-           subpool));
-
-  return SVN_NO_ERROR;
+  return svn_wc_queue_committed2(btn->queue, item->path, adm_access,
+                                 loop_recurse, item->incoming_prop_changes,
+                                 remove_lock, !btn->keep_changelists,
+                                 apr_hash_get(btn->checksums,
+                                              item->path,
+                                              APR_HASH_KEY_STRING),
+                                 subpool);
 }
 
 
@@ -1309,8 +1255,6 @@ check_nonrecursive_dir_delete(void *baton, void *this_item, apr_pool_t *pool)
   /* ### TODO(sd): This check is slightly too strict.  It should be
      ### possible to:
      ###
-     ###   * delete an empty directory when depth==svn_depth_empty;
-     ###
      ###   * delete a directory containing only files when
      ###     depth==svn_depth_files;
      ###
@@ -1336,9 +1280,17 @@ check_nonrecursive_dir_delete(void *baton, void *this_item, apr_pool_t *pool)
           SVN_ERR(svn_wc_status2(&status, target, adm_access, pool));
           if (status->text_status == svn_wc_status_deleted ||
               status->text_status == svn_wc_status_replaced)
-            return svn_error_create(SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+            {
+              apr_hash_t* entries;
+
+              SVN_ERR(svn_wc_entries_read(&entries, adm_access, TRUE, pool));
+
+              if (apr_hash_count(entries) > 1)
+                return svn_error_create(SVN_ERR_UNSUPPORTED_FEATURE, NULL,
                                     _("Cannot non-recursively commit a "
-                                      "directory deletion"));
+                                      "directory deletion of a directory "
+                                      "with child nodes"));
+            }
         }
     }
   return SVN_NO_ERROR;
@@ -1366,13 +1318,17 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
   apr_array_header_t *dirs_to_lock;
   apr_array_header_t *dirs_to_lock_recursive;
   svn_boolean_t lock_base_dir_recursive = FALSE;
-  apr_hash_t *committables, *lock_tokens, *tempfiles = NULL, *digests;
+  apr_hash_t *committables;
+  apr_hash_t *lock_tokens;
+  apr_hash_t *tempfiles = NULL;
+  apr_hash_t *checksums;
   svn_wc_adm_access_t *base_dir_access;
   apr_array_header_t *commit_items;
   svn_error_t *cmt_err = SVN_NO_ERROR, *unlock_err = SVN_NO_ERROR;
   svn_error_t *bump_err = SVN_NO_ERROR, *cleanup_err = SVN_NO_ERROR;
   svn_boolean_t commit_in_progress = FALSE;
-  const char *display_dir = "";
+  const char *current_dir = "";
+  const char *notify_prefix;
   int i;
 
   /* Committing URLs doesn't make sense, so error if it's tried. */
@@ -1389,6 +1345,10 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
   SVN_ERR(svn_path_condense_targets(&base_dir, &rel_targets, targets,
                                     depth == svn_depth_infinity, pool));
 
+  /* No targets means nothing to commit, so just return. */
+  if (! base_dir)
+    goto cleanup;
+
   /* When svn_path_condense_targets() was written, we didn't have real
    * depths, we just had recursive / nonrecursive.
    *
@@ -1440,19 +1400,18 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
    */
   if (depth == svn_depth_files || depth == svn_depth_immediates)
     {
-      const char *rel_target;
       for (i = 0; i < rel_targets->nelts; ++i)
         {
-          rel_target = APR_ARRAY_IDX(rel_targets, i, const char *);
+          const char *rel_target = APR_ARRAY_IDX(rel_targets, i, const char *);
+
           if (rel_target[0] == '\0')
-            lock_base_dir_recursive = TRUE;
+            {
+              lock_base_dir_recursive = TRUE;
+              break;
+            }
         }
     }
 
-  /* No targets means nothing to commit, so just return. */
-  if (! base_dir)
-    goto cleanup;
-
   /* Prepare an array to accumulate dirs to lock */
   dirs_to_lock = apr_array_make(pool, 1, sizeof(target));
   dirs_to_lock_recursive = apr_array_make(pool, 1, sizeof(target));
@@ -1509,7 +1468,6 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
 
       for (i = 0; i < rel_targets->nelts; i++)
         {
-          const char *parent_dir, *name;
           svn_node_kind_t kind;
 
           svn_pool_clear(subpool);
@@ -1543,20 +1501,21 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
              Do nothing if target is already the base_dir. */
           if (strcmp(target, base_dir) != 0)
             {
-              svn_path_split(target, &parent_dir, &name, subpool);
-
-              target = parent_dir;
+              target = svn_dirent_dirname(target, subpool);
 
               while (strcmp(target, base_dir) != 0)
                 {
-                  if ((target[0] == '\0') ||
-                      svn_dirent_is_root(target, strlen(target))
-                     )
-                    abort();
+                  const char *parent_dir;
 
                   APR_ARRAY_PUSH(dirs_to_lock,
                                  const char *) = apr_pstrdup(pool, target);
-                  target = svn_path_dirname(target, subpool);
+
+                  parent_dir = svn_dirent_dirname(target, subpool);
+
+                  if (strcmp(parent_dir, target) == 0)
+                    break; /* Reached root directory */
+                  
+                  target = parent_dir;
                 }
             }
         }
@@ -1688,9 +1647,9 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
                                      pool)))
     goto cleanup;
 
-  if ((cmt_err = get_ra_editor(&ra_session, NULL,
+  if ((cmt_err = get_ra_editor(&ra_session,
                                &editor, &edit_baton, ctx,
-                               base_url, base_dir, base_dir_access, log_msg, 
+                               base_url, base_dir, base_dir_access, log_msg,
                                commit_items, revprop_table, commit_info_p,
                                TRUE, lock_tokens, keep_locks, pool)))
     goto cleanup;
@@ -1698,17 +1657,18 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
   /* Make a note that we have a commit-in-progress. */
   commit_in_progress = TRUE;
 
-  /* Determine prefix to strip from the commit notify messages */
-  if ((cmt_err = svn_path_get_absolute(&display_dir,
-                                       display_dir, pool)))
+  if ((cmt_err = svn_path_get_absolute(&current_dir,
+                                       current_dir, pool)))
     goto cleanup;
-  display_dir = svn_path_get_longest_ancestor(display_dir, base_dir, pool);
+
+  /* Determine prefix to strip from the commit notify messages */
+  notify_prefix = svn_path_get_longest_ancestor(current_dir, base_dir, pool);
 
   /* Perform the commit. */
   cmt_err = svn_client__do_commit(base_url, commit_items, base_dir_access,
                                   editor, edit_baton,
-                                  display_dir,
-                                  &tempfiles, &digests, ctx, pool);
+                                  notify_prefix,
+                                  &tempfiles, &checksums, ctx, pool);
 
   /* Handle a successful commit. */
   if ((! cmt_err)
@@ -1722,7 +1682,7 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
       btn.base_dir_access = base_dir_access;
       btn.keep_changelists = keep_changelists;
       btn.keep_locks = keep_locks;
-      btn.digests = digests;
+      btn.checksums = checksums;
 
       /* Make a note that our commit is finished. */
       commit_in_progress = FALSE;
@@ -1733,7 +1693,7 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
       if (bump_err)
         goto cleanup;
 
-      assert(*commit_info_p);
+      SVN_ERR_ASSERT(*commit_info_p);
       bump_err
         = svn_wc_process_committed_queue(queue, base_dir_access,
                                          (*commit_info_p)->revision,
@@ -1743,7 +1703,7 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
     }
 
   /* Sleep to ensure timestamp integrity. */
-  svn_sleep_for_timestamps();
+  svn_io_sleep_for_timestamps(base_dir, pool);
 
  cleanup:
   /* Abort the commit if it is still in progress. */
@@ -1757,61 +1717,16 @@ svn_client_commit4(svn_commit_info_t **commit_info_p,
      clean-up. */
   if (! bump_err)
     {
-      unlock_err = svn_wc_adm_close(base_dir_access);
+      unlock_err = svn_wc_adm_close2(base_dir_access, pool);
 
       if (! unlock_err)
         cleanup_err = remove_tmpfiles(tempfiles, pool);
     }
 
-  /* As per our promise, if *commit_info_p isn't set, provide a default where 
+  /* As per our promise, if *commit_info_p isn't set, provide a default where
      rev = SVN_INVALID_REVNUM. */
   if (! *commit_info_p)
     *commit_info_p = svn_create_commit_info(pool);
 
   return reconcile_errors(cmt_err, unlock_err, bump_err, cleanup_err, pool);
 }
-
-svn_error_t *
-svn_client_commit3(svn_commit_info_t **commit_info_p,
-                   const apr_array_header_t *targets,
-                   svn_boolean_t recurse,
-                   svn_boolean_t keep_locks,
-                   svn_client_ctx_t *ctx,
-                   apr_pool_t *pool)
-{
-  svn_depth_t depth = SVN_DEPTH_INFINITY_OR_EMPTY(recurse);
-
-  return svn_client_commit4(commit_info_p, targets, depth, keep_locks,
-                            FALSE, NULL, NULL, ctx, pool);
-}
-
-svn_error_t *
-svn_client_commit2(svn_client_commit_info_t **commit_info_p,
-                   const apr_array_header_t *targets,
-                   svn_boolean_t recurse,
-                   svn_boolean_t keep_locks,
-                   svn_client_ctx_t *ctx,
-                   apr_pool_t *pool)
-{
-  svn_commit_info_t *commit_info = NULL;
-  svn_error_t *err;
-
-  err = svn_client_commit3(&commit_info, targets, recurse, keep_locks,
-                           ctx, pool);
-  /* These structs have the same layout for the common fields. */
-  *commit_info_p = (svn_client_commit_info_t *) commit_info;
-  return err;
-}
-
-svn_error_t *
-svn_client_commit(svn_client_commit_info_t **commit_info_p,
-                  const apr_array_header_t *targets,
-                  svn_boolean_t nonrecursive,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
-{
-  return svn_client_commit2(commit_info_p, targets,
-                            nonrecursive ? FALSE : TRUE,
-                            TRUE,
-                            ctx, pool);
-}
