@@ -1,7 +1,7 @@
 /* reps-strings.h : interpreting representations with respect to strings
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004, 2009 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -18,8 +18,8 @@
 #ifndef SVN_LIBSVN_FS_REPS_STRINGS_H
 #define SVN_LIBSVN_FS_REPS_STRINGS_H
 
-#define APU_WANT_DB
-#include <apu_want.h>
+#define SVN_WANT_BDB
+#include "svn_private_config.h"
 
 #include "svn_io.h"
 #include "svn_fs.h"
@@ -76,16 +76,24 @@ svn_error_t *svn_fs_base__rep_contents_size(svn_filesize_t *size_p,
                                             apr_pool_t *pool);
 
 
-/* Put into DIGEST the MD5 checksum for REP_KEY in FS, as part of TRAIL.
-   This is the prerecorded checksum for the rep's contents' fulltext.
-   If no checksum is available, do not calculate one dynamically, just
-   put all 0's into DIGEST.  (By convention, the all-zero checksum is
-   considered to match any checksum.) */
-svn_error_t *svn_fs_base__rep_contents_checksum(unsigned char digest[],
-                                                svn_fs_t *fs,
-                                                const char *rep_key,
-                                                trail_t *trail,
-                                                apr_pool_t *pool);
+/* If MD5_CHECKSUM is non-NULL, set *MD5_CHECKSUM to the MD5 checksum
+   for REP_KEY in FS, as part of TRAIL.
+
+   If SHA1_CHECKSUM is non-NULL, set *SHA1_CHECKSUM to the SHA1
+   checksum for REP_KEY in FS, as part of TRAIL.
+
+   These are the prerecorded checksums for the rep's contents'
+   fulltext.  If one or both of the checksums is not stored, do not
+   calculate one dynamically, just put NULL into the respective return
+   value.  (By convention, the NULL checksum is considered to match
+   any checksum.) */
+svn_error_t *
+svn_fs_base__rep_contents_checksums(svn_checksum_t **md5_checksum,
+                                    svn_checksum_t **sha1_checksum,
+                                    svn_fs_t *fs,
+                                    const char *rep_key,
+                                    trail_t *trail,
+                                    apr_pool_t *pool);
 
 
 /* Set STR->data to the contents of REP_KEY in FS, and STR->len to the
