@@ -5,7 +5,7 @@
 #  See http://subversion.tigris.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -20,11 +20,21 @@ import os
 import shutil  # for rmtree()
 import re
 import stat    # for ST_MODE
+import subprocess
 import copy    # for deepcopy()
 import time    # for time()
 import traceback # for print_exc()
 import threading
-import Queue
+try:
+  # Python >=3.0
+  import queue
+  from urllib.parse import quote as urllib_parse_quote
+  from urllib.parse import unquote as urllib_parse_unquote
+except ImportError:
+  # Python <3.0
+  import Queue as queue
+  from urllib import quote as urllib_parse_quote
+  from urllib import unquote as urllib_parse_unquote
 
 import getopt
 try:
@@ -96,14 +106,8 @@ else:
   _exe = ''
   _bat = ''
 
-try:
-  from popen2 import Popen3
-  platform_with_popen3_class = True
-except ImportError:
-  platform_with_popen3_class = False
-
 # The location of our mock svneditor script.
-if sys.platform == 'win32':
+if windows:
   svneditor_script = os.path.join(sys.path[0], 'svneditor.bat')
 else:
   svneditor_script = os.path.join(sys.path[0], 'svneditor.py')
@@ -116,6 +120,23 @@ wc_passwd = 'rayjandom'
 # scenarios
 wc_author2 = 'jconstant' # use the same password as wc_author
 
+# Set C locale for command line programs
+os.environ['LC_ALL'] = 'C'
+
+# This function mimics the Python 2.3 urllib function of the same name.
+def pathname2url(path):
+  """Convert the pathname PATH from the local syntax for a path to the form
+  used in the path component of a URL. This does not produce a complete URL.
+  The return value will already be quoted using the quote() function."""
+  return urllib_parse_quote(path.replace('\\', '/'))
+
+# This function mimics the Python 2.3 urllib function of the same name.
+def url2pathname(path):
+  """Convert the path component PATH from an encoded URL to the local syntax
+  for a path. This does not accept a complete URL. This function uses
+  unquote() to decode PATH."""
+  return os.path.normpath(urllib_parse_unquote(path))
+
 ######################################################################
 # Global variables set during option parsing.  These should not be used
 # until the variable command_line_parsed has been set to True, as is
@@ -149,10 +170,23 @@ cleanup_mode = False
 # Global variable indicating if svnserve should use Cyrus SASL
 enable_sasl = False
 
+# Global variable indicating that SVNKit binaries should be used
+use_jsvn = False
+
 # Global variable indicating which DAV library, if any, is in use
 # ('neon', 'serf')
 http_library = None
 
+# Global variable: Number of shards to use in FSFS
+# 'None' means "use FSFS's default"
+fsfs_sharding = None
+
+# Global variable: automatically pack FSFS repositories after every commit
+fsfs_packing = None
+
+# Configuration file (copied into FSFS fsfs.conf).
+config_file = None
+
 # Global variable indicating what the minor version of the server
 # tested against is (4 for 1.4.x, for example).
 server_minor_version = 5
@@ -162,9 +196,7 @@ server_minor_version = 5
 is_child_process = False
 
 # Global URL to testing area.  Default to ra_local, current working dir.
-test_area_url = file_scheme_prefix + os.path.abspath(os.getcwd())
-if windows:
-  test_area_url = test_area_url.replace('\\', '/')
+test_area_url = file_scheme_prefix + pathname2url(os.path.abspath(os.getcwd()))
 
 # Location to the pristine repository, will be calculated from test_area_url
 # when we know what the user specified for --url.
@@ -241,9 +273,9 @@ def wrap_ex(func):
       if ex.__class__ != Failure or ex.args:
         ex_args = str(ex)
         if ex_args:
-          print 'EXCEPTION: %s: %s' % (ex.__class__.__name__, ex_args)
+          print('EXCEPTION: %s: %s' % (ex.__class__.__name__, ex_args))
         else:
-          print 'EXCEPTION:', ex.__class__.__name__
+          print('EXCEPTION: %s' % ex.__class__.__name__)
   return w
 
 def setup_development_mode():
@@ -264,7 +296,7 @@ def setup_development_mode():
         'run_and_verify_diff_summarize',
         'run_and_verify_diff_summarize_xml',
         'run_and_validate_lock']
-  
+
   for func in l:
     setattr(actions, func, wrap_ex(getattr(actions, func)))
 
@@ -272,7 +304,7 @@ def get_admin_name():
   "Return name of SVN administrative subdirectory."
 
   if (windows or sys.platform == 'cygwin') \
-      and os.environ.has_key('SVN_ASP_DOT_NET_HACK'):
+      and 'SVN_ASP_DOT_NET_HACK' in os.environ:
     return '_svn'
   else:
     return '.svn'
@@ -304,9 +336,20 @@ def get_svnserve_conf_file_path(repo_dir):
 
   return os.path.join(repo_dir, "conf", "svnserve.conf")
 
-# Run any binary, logging the command line (TODO: and return code)
+def get_fsfs_conf_file_path(repo_dir):
+  "Return the path of the fsfs.conf file in REPO_DIR."
+
+  return os.path.join(repo_dir, "db", "fsfs.conf")
+
+def get_fsfs_format_file_path(repo_dir):
+  "Return the path of the format file in REPO_DIR."
+
+  return os.path.join(repo_dir, "db", "format")
+
+# Run any binary, logging the command line and return code
 def run_command(command, error_expected, binary_mode=0, *varargs):
-  """Run COMMAND with VARARGS; return stdout, stderr as lists of lines.
+  """Run COMMAND with VARARGS; return exit code as int; stdout, stderr
+  as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed."""
 
   return run_command_stdin(command, error_expected, binary_mode,
@@ -338,31 +381,70 @@ def _quote_arg(arg):
       arg = arg.replace('$', '\$')
     return '"%s"' % (arg,)
 
-def open_pipe(command, mode):
-  """Opens a popen3 pipe to COMMAND in MODE.
+def open_pipe(command, stdin=None, stdout=None, stderr=None):
+  """Opens a subprocess.Popen pipe to COMMAND using STDIN,
+  STDOUT, and STDERR.
 
   Returns (infile, outfile, errfile, waiter); waiter
   should be passed to wait_on_pipe."""
-  if platform_with_popen3_class:
-    kid = Popen3(command, True)
-    return kid.tochild, kid.fromchild, kid.childerr, (kid, command)
-  else:
-    inf, outf, errf = os.popen3(command, mode)
-    return inf, outf, errf, None
+  command = [str(x) for x in command]
 
-def wait_on_pipe(waiter, stdout_lines, stderr_lines):
+  # On Windows subprocess.Popen() won't accept a Python script as
+  # a valid program to execute, rather it wants the Python executable.
+  if (sys.platform == 'win32') and (command[0].endswith('.py')):
+    command.insert(0, sys.executable)
+
+  # Quote only the arguments on Windows.  Later versions of subprocess,
+  # 2.5.2+ confirmed, don't require this quoting, but versions < 2.4.3 do.
+  if sys.platform == 'win32':
+    args = command[1:]
+    args = ' '.join([_quote_arg(x) for x in args])
+    command = command[0] + ' ' + args
+    command_string = command
+  else:
+    command_string = ' '.join(command)
+
+  if not stdin:
+    stdin = subprocess.PIPE
+  if not stdout:
+    stdout = subprocess.PIPE
+  if not stderr:
+    stderr = subprocess.PIPE
+
+  p = subprocess.Popen(command,
+                       stdin=stdin,
+                       stdout=stdout,
+                       stderr=stderr,
+                       close_fds=not windows)
+  return p.stdin, p.stdout, p.stderr, (p, command_string)
+
+def wait_on_pipe(waiter, binary_mode, stdin=None):
   """Waits for KID (opened with open_pipe) to finish, dying
-  if it does.  Uses STDOUT_LINES and STDERR_LINES for error message
-  if kid fails.  Returns kid's exit code."""
+  if it does.  If kid fails create an error message containing
+  any stdout and stderr from the kid.  Returns kid's exit code,
+  stdout and stderr (the latter two as lists)."""
   if waiter is None:
     return
 
-  kid, command = waiter
+  kid, command_string = waiter
+  stdout, stderr = kid.communicate(stdin)
+  exit_code = kid.returncode
 
-  wait_code = kid.wait()
+  # Normalize Windows line endings if in text mode.
+  if windows and not binary_mode:
+    stdout = stdout.replace('\r\n', '\n')
+    stderr = stderr.replace('\r\n', '\n')
+
+  # Convert output strings to lists.
+  stdout_lines = stdout.splitlines(True)
+  stderr_lines = stderr.splitlines(True)
+
+  if exit_code < 0:
+    if not windows:
+      exit_signal = os.WTERMSIG(-exit_code)
+    else:
+      exit_signal = exit_code
 
-  if os.WIFSIGNALED(wait_code):
-    exit_signal = os.WTERMSIG(wait_code)
     if stdout_lines is not None:
       sys.stdout.write("".join(stdout_lines))
     if stderr_lines is not None:
@@ -370,42 +452,34 @@ def wait_on_pipe(waiter, stdout_lines, stderr_lines):
     if verbose_mode:
       # show the whole path to make it easier to start a debugger
       sys.stderr.write("CMD: %s terminated by signal %d\n"
-                       % (command, exit_signal))
+                       % (command_string, exit_signal))
     raise SVNProcessTerminatedBySignal
   else:
-    exit_code = os.WEXITSTATUS(wait_code)
     if exit_code and verbose_mode:
-      sys.stderr.write("CMD: %s exited with %d\n" % (command, exit_code))
-    return exit_code
+      sys.stderr.write("CMD: %s exited with %d\n"
+                       % (command_string, exit_code))
+    return stdout_lines, stderr_lines, exit_code
 
 # Run any binary, supplying input text, logging the command line
 def spawn_process(command, binary_mode=0,stdin_lines=None, *varargs):
-  args = ' '.join(map(_quote_arg, varargs))
-
   # Log the command line
   if verbose_mode and not command.endswith('.py'):
-    print 'CMD:', os.path.basename(command) + ' ' + args,
+    sys.stdout.write('CMD: %s %s ' % (os.path.basename(command),
+                                      ' '.join([_quote_arg(x) for x in varargs])))
+    sys.stdout.flush()
 
-  if binary_mode:
-    mode = 'b'
-  else:
-    mode = 't'
-
-  infile, outfile, errfile, kid = open_pipe(command + ' ' + args, mode)
+  infile, outfile, errfile, kid = open_pipe([command] + list(varargs))
 
   if stdin_lines:
-    map(infile.write, stdin_lines)
+    for x in stdin_lines:
+      infile.write(x)
 
+  stdout_lines, stderr_lines, exit_code = wait_on_pipe(kid, binary_mode)
   infile.close()
 
-  stdout_lines = outfile.readlines()
-  stderr_lines = errfile.readlines()
-
   outfile.close()
   errfile.close()
 
-  exit_code = wait_on_pipe(kid, stdout_lines, stderr_lines)
-
   return exit_code, stdout_lines, stderr_lines
 
 def run_command_stdin(command, error_expected, binary_mode=0,
@@ -415,7 +489,7 @@ def run_command_stdin(command, error_expected, binary_mode=0,
   should not be very large, as if the program outputs more than the OS
   is willing to buffer, this will deadlock, with both Python and
   COMMAND waiting to write to each other for ever.
-  Return stdout, stderr as lists of lines.
+  Return exit code as int; stdout, stderr as lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed."""
 
   if verbose_mode:
@@ -428,13 +502,19 @@ def run_command_stdin(command, error_expected, binary_mode=0,
 
   if verbose_mode:
     stop = time.time()
-    print '<TIME = %.6f>' % (stop - start)
+    print('<TIME = %.6f>' % (stop - start))
+    for x in stdout_lines:
+      sys.stdout.write(x)
+    for x in stderr_lines:
+      sys.stdout.write(x)
 
   if (not error_expected) and (stderr_lines):
-    map(sys.stdout.write, stderr_lines)
+    if not verbose_mode:
+      for x in stderr_lines:
+        sys.stdout.write(x)
     raise Failure
 
-  return stdout_lines, stderr_lines
+  return exit_code, stdout_lines, stderr_lines
 
 def create_config_dir(cfgdir, config_contents=None, server_contents=None):
   "Create config directories and files"
@@ -451,20 +531,25 @@ def create_config_dir(cfgdir, config_contents=None, server_contents=None):
   if config_contents is None:
     config_contents = """
 #
+[auth]
+password-stores =
+
 [miscellany]
 interactive-conflicts = false
 """
 
   # define default server file contents if none provided
   if server_contents is None:
+    http_library_str = ""
     if http_library:
-      server_contents = """
+      http_library_str = "http-library=%s" % (http_library)
+    server_contents = """
 #
 [global]
-http-library=%s
-""" % (http_library)
-    else:
-      server_contents = "#\n"
+%s
+store-plaintext-passwords=yes
+store-passwords=yes
+""" % (http_library_str)
 
   file_write(cfgfile_cfg, config_contents)
   file_write(cfgfile_srv, server_contents)
@@ -486,7 +571,8 @@ def _with_auth(args):
 
 # For running subversion and returning the output
 def run_svn(error_expected, *varargs):
-  """Run svn with VARARGS; return stdout, stderr as lists of lines.
+  """Run svn with VARARGS; return exit code as int; stdout, stderr as
+  lists of lines.
   If ERROR_EXPECTED is None, any stderr also will be printed.  If
   you're just checking that something does/doesn't come out of
   stdout/stderr, you might want to use actions.run_and_verify_svn()."""
@@ -495,20 +581,24 @@ def run_svn(error_expected, *varargs):
 
 # For running svnadmin.  Ignores the output.
 def run_svnadmin(*varargs):
-  "Run svnadmin with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnadmin with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnadmin_binary, 1, 0, *varargs)
 
 # For running svnlook.  Ignores the output.
 def run_svnlook(*varargs):
-  "Run svnlook with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnlook with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnlook_binary, 1, 0, *varargs)
 
 def run_svnsync(*varargs):
-  "Run svnsync with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnsync with VARARGS, returns exit code as int; stdout, stderr as
+  list of lines."""
   return run_command(svnsync_binary, 1, 0, *(_with_config_dir(varargs)))
 
 def run_svnversion(*varargs):
-  "Run svnversion with VARARGS, returns stdout, stderr as list of lines."
+  """Run svnversion with VARARGS, returns exit code as int; stdout, stderr
+  as list of lines."""
   return run_command(svnversion_binary, 1, 0, *varargs)
 
 # Chmod recursively on a whole subtree
@@ -595,7 +685,8 @@ def create_repos(path):
     opts += ("--pre-1.5-compatible",)
   if fs_type is not None:
     opts += ("--fs-type=" + fs_type,)
-  stdout, stderr = run_command(svnadmin_binary, 1, 0, "create", path, *opts)
+  exit_code, stdout, stderr = run_command(svnadmin_binary, 1, 0, "create",
+                                          path, *opts)
 
   # Skip tests if we can't create the repository.
   if stderr:
@@ -616,6 +707,50 @@ def create_repos(path):
     file_append(get_svnserve_conf_file_path(path), "password-db = passwd\n")
     file_append(os.path.join(path, "conf", "passwd"),
                 "[users]\njrandom = rayjandom\njconstant = rayjandom\n");
+
+  if fs_type is None or fs_type == 'fsfs':
+    # fsfs.conf file
+    if config_file is not None:
+      shutil.copy(config_file, get_fsfs_conf_file_path(path))
+
+    # format file
+    if fsfs_sharding is not None:
+      def transform_line(line):
+        if line.startswith('layout '):
+          if fsfs_sharding > 0:
+            line = 'layout sharded %d' % fsfs_sharding
+          else:
+            line = 'layout linear'
+        return line
+
+      # read it
+      format_file_path = get_fsfs_format_file_path(path)
+      contents = file_read(format_file_path, 'rb')
+
+      # tweak it
+      new_contents = "".join([transform_line(line) + "\n"
+                              for line in contents.split("\n")])
+      if new_contents[-1] == "\n":
+        # we don't currently allow empty lines (\n\n) in the format file.
+        new_contents = new_contents[:-1]
+
+      # replace it
+      os.chmod(format_file_path, 0666)
+      file_write(format_file_path, new_contents, 'wb')
+
+    # post-commit
+    # Note that some tests (currently only commit_tests) create their own
+    # post-commit hooks, which would override this one. :-(
+    if fsfs_packing:
+      # some tests chdir.
+      abs_path = os.path.abspath(path)
+      create_python_hook_script(get_post_commit_hook_path(abs_path),
+          "import subprocess\n"
+          "import sys\n"
+          "command = %s\n"
+          "sys.exit(subprocess.Popen(command).wait())\n"
+          % repr([svnadmin_binary, 'pack', abs_path]))
+
   # make the repos world-writeable, for mod_dav_svn's sake.
   chmod_tree(path, 0666, 0666)
 
@@ -626,65 +761,64 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 1):
   # Do an svnadmin dump|svnadmin load cycle. Print a fake pipe command so that
   # the displayed CMDs can be run by hand
   create_repos(dst_path)
-  dump_args = ' dump "' + src_path + '"'
-  load_args = ' load "' + dst_path + '"'
+  dump_args = ['dump', src_path]
+  load_args = ['load', dst_path]
 
   if ignore_uuid:
-    load_args = load_args + " --ignore-uuid"
+    load_args = load_args + ['--ignore-uuid']
   if verbose_mode:
-    print 'CMD:', os.path.basename(svnadmin_binary) + dump_args, \
-          '|', os.path.basename(svnadmin_binary) + load_args,
+    sys.stdout.write('CMD: %s%s | %s%s ' % (os.path.basename(svnadmin_binary),
+                                            ' '.join(dump_args),
+                                            os.path.basename(svnadmin_binary),
+                                            ' '.join(load_args)))
+    sys.stdout.flush()
   start = time.time()
 
-  dump_in, dump_out, dump_err, dump_kid = \
-           open_pipe(svnadmin_binary + dump_args, 'b')
-  load_in, load_out, load_err, load_kid = \
-           open_pipe(svnadmin_binary + load_args, 'b')
+  dump_in, dump_out, dump_err, dump_kid = open_pipe(
+    [svnadmin_binary] + dump_args)
+  load_in, load_out, load_err, load_kid = open_pipe(
+    [svnadmin_binary] + load_args,
+    stdin=dump_out) # Attached to dump_kid
+
   stop = time.time()
   if verbose_mode:
-    print '<TIME = %.6f>' % (stop - start)
+    print('<TIME = %.6f>' % (stop - start))
 
-  while 1:
-    data = dump_out.read(1024*1024)  # Arbitrary buffer size
-    if data == "":
-      break
-    load_in.write(data)
-  load_in.close() # Tell load we are done
+  load_stdout, load_stderr, load_exit_code = wait_on_pipe(load_kid, True)
+  dump_stdout, dump_stderr, dump_exit_code = wait_on_pipe(dump_kid, True)
 
-  dump_lines = dump_err.readlines()
-  load_lines = load_out.readlines()
   dump_in.close()
   dump_out.close()
   dump_err.close()
+  #load_in is dump_out so it's already closed.
   load_out.close()
   load_err.close()
-  # Wait on the pipes; ignore return code.
-  wait_on_pipe(dump_kid, None, dump_lines)
-  wait_on_pipe(load_kid, load_lines, None)
 
   dump_re = re.compile(r'^\* Dumped revision (\d+)\.\r?$')
   expect_revision = 0
-  for dump_line in dump_lines:
+  for dump_line in dump_stderr:
     match = dump_re.match(dump_line)
     if not match or match.group(1) != str(expect_revision):
-      print 'ERROR:  dump failed:', dump_line,
+      sys.stdout.write('ERROR:  dump failed: %s ' % dump_line)
+      sys.stdout.flush()
       raise SVNRepositoryCopyFailure
     expect_revision += 1
   if expect_revision != head_revision + 1:
-    print 'ERROR:  dump failed; did not see revision', head_revision
+    print('ERROR:  dump failed; did not see revision %s' % head_revision)
     raise SVNRepositoryCopyFailure
 
   load_re = re.compile(r'^------- Committed revision (\d+) >>>\r?$')
   expect_revision = 1
-  for load_line in load_lines:
+  for load_line in load_stdout:
     match = load_re.match(load_line)
     if match:
       if match.group(1) != str(expect_revision):
-        print 'ERROR:  load failed:', load_line,
+        sys.stdout.write('ERROR:  load failed: %s ' % load_line)
+        sys.stdout.flush()
         raise SVNRepositoryCopyFailure
       expect_revision += 1
   if expect_revision != head_revision + 1:
-    print 'ERROR:  load failed; did not see revision', head_revision
+    print('ERROR:  load failed; did not see revision %s' % head_revision)
     raise SVNRepositoryCopyFailure
 
 
@@ -703,7 +837,7 @@ def create_python_hook_script (hook_path, hook_script_code):
   """Create a Python hook script at HOOK_PATH with the specified
      HOOK_SCRIPT_CODE."""
 
-  if sys.platform == 'win32':
+  if windows:
     # Use an absolute path since the working directory is not guaranteed
     hook_path = os.path.abspath(hook_path)
     # Fill the python file.
@@ -808,9 +942,21 @@ def _check_command_line_parsed():
   if not command_line_parsed:
     raise Failure("Condition cannot be tested until command line is parsed")
 
+def is_not_serf():
+  _check_command_line_parsed()
+  return not (http_library == "serf")
+
 def is_ra_type_dav():
   _check_command_line_parsed()
   return test_area_url.startswith('http')
+  
+def is_ra_type_dav_neon():
+  _check_command_line_parsed()
+  return test_area_url.startswith('http') and not(http_library == "serf")
+  
+def is_ra_type_dav_serf():
+  _check_command_line_parsed()
+  return test_area_url.startswith('http') and (http_library == "serf")
 
 def is_ra_type_svn():
   _check_command_line_parsed()
@@ -884,11 +1030,11 @@ class Sandbox:
     self.wc_dir = os.path.join(general_wc_dir, self.name)
     if not read_only:
       self.repo_dir = os.path.join(general_repo_dir, self.name)
-      self.repo_url = test_area_url + '/' + self.repo_dir
+      self.repo_url = test_area_url + '/' + pathname2url(self.repo_dir)
     else:
       self.repo_dir = pristine_dir
       self.repo_url = pristine_url
-      
+
     ### TODO: Move this into to the build() method
     # For dav tests we need a single authz file which must be present,
     # so we recreate it each time a sandbox is created with some default
@@ -906,8 +1052,6 @@ class Sandbox:
     elif self.repo_url.startswith("svn"):
       self.authz_file = os.path.join(self.repo_dir, "conf", "authz")
 
-    if windows:
-      self.repo_url = self.repo_url.replace('\\', '/')
     self.test_paths = [self.wc_dir, self.repo_dir]
 
   def clone_dependent(self, copy_wc=False):
@@ -940,9 +1084,7 @@ class Sandbox:
 
   def add_repo_path(self, suffix, remove=1):
     path = os.path.join(general_repo_dir, self.name)  + '.' + suffix
-    url  = test_area_url + '/' + path
-    if windows:
-      url = url.replace('\\', '/')
+    url  = test_area_url + '/' + pathname2url(path)
     self.add_test_path(path, remove)
     return path, url
 
@@ -974,14 +1116,14 @@ def _cleanup_deferred_test_paths():
 def _cleanup_test_path(path, retrying=None):
   if verbose_mode:
     if retrying:
-      print "CLEANUP: RETRY:", path
+      print("CLEANUP: RETRY: %s" % path)
     else:
-      print "CLEANUP:", path
+      print("CLEANUP: %s" % path)
   try:
     safe_rmtree(path)
   except:
     if verbose_mode:
-      print "WARNING: cleanup failed, will try again later"
+      print("WARNING: cleanup failed, will try again later")
     _deferred_test_paths.append(path)
 
 class TestSpawningThread(threading.Thread):
@@ -997,7 +1139,7 @@ class TestSpawningThread(threading.Thread):
     while True:
       try:
         next_index = self.queue.get_nowait()
-      except Queue.Empty:
+      except queue.Empty:
         return
 
       self.run_one(next_index)
@@ -1025,10 +1167,6 @@ class TestSpawningThread(threading.Thread):
       args.append('--server-minor-version=' + str(server_minor_version))
 
     result, stdout_lines, stderr_lines = spawn_process(command, 1, None, *args)
-    # don't trust the exitcode, will not be correct on Windows
-    if filter(lambda x: x.startswith('FAIL: ') or x.startswith('XPASS: '),
-              stdout_lines):
-      result = 1
     self.results.append((index, result, stdout_lines, stderr_lines))
     sys.stdout.write('.')
     sys.stdout.flush()
@@ -1042,14 +1180,14 @@ class TestRunner:
     self.index = index
 
   def list(self):
-    print " %2d     %-5s  %s" % (self.index,
+    print(" %2d     %-5s  %s" % (self.index,
                                  self.pred.list_mode(),
-                                 self.pred.get_description())
+                                 self.pred.get_description()))
     self.pred.check_description()
 
   def _print_name(self):
-    print os.path.basename(sys.argv[0]), str(self.index) + ":", \
-          self.pred.get_description()
+    print("%s %s: %s" % (os.path.basename(sys.argv[0]), str(self.index),
+          self.pred.get_description()))
     self.pred.check_description()
 
   def run(self):
@@ -1071,15 +1209,24 @@ class TestRunner:
     # Tests that want to use an editor should invoke svntest.main.use_editor.
     os.environ['SVN_EDITOR'] = ''
     os.environ['SVNTEST_EDITOR_FUNC'] = ''
+
+    if use_jsvn:
+      # Set this SVNKit specific variable to the current test (test name plus
+      # its index) being run so that SVNKit daemon could use this test name
+      # for its separate log file
+     os.environ['SVN_CURRENT_TEST'] = os.path.basename(sys.argv[0]) + "_" + \
+                                      str(self.index)
+
     actions.no_sleep_for_timestamps()
 
     saved_dir = os.getcwd()
     try:
-      rc = apply(self.pred.run, (), kw)
+      rc = self.pred.run(**kw)
       if rc is not None:
-        print 'STYLE ERROR in',
+        sys.stdout.write('STYLE ERROR in ')
+        sys.stdout.flush()
         self._print_name()
-        print 'Test driver returned a status code.'
+        print('Test driver returned a status code.')
         sys.exit(255)
       result = 0
     except Skip, ex:
@@ -1093,28 +1240,30 @@ class TestRunner:
       if ex.__class__ != Failure or ex.args:
         ex_args = str(ex)
         if ex_args:
-          print 'EXCEPTION: %s: %s' % (ex.__class__.__name__, ex_args)
+          print('EXCEPTION: %s: %s' % (ex.__class__.__name__, ex_args))
         else:
-          print 'EXCEPTION:', ex.__class__.__name__
+          print('EXCEPTION: %s' % ex.__class__.__name__)
       traceback.print_exc(file=sys.stdout)
     except KeyboardInterrupt:
-      print 'Interrupted'
+      print('Interrupted')
       sys.exit(0)
     except SystemExit, ex:
-      print 'EXCEPTION: SystemExit(%d), skipping cleanup' % ex.code
-      print ex.code and 'FAIL: ' or 'PASS: ',
+      print('EXCEPTION: SystemExit(%d), skipping cleanup' % ex.code)
+      sys.stdout.write(ex.code and 'FAIL:  ' or 'PASS:  ')
+      sys.stdout.flush()
       self._print_name()
       raise
     except:
       result = 1
-      print 'UNEXPECTED EXCEPTION:'
+      print('UNEXPECTED EXCEPTION:')
       traceback.print_exc(file=sys.stdout)
 
     os.chdir(saved_dir)
     result = self.pred.convert_result(result)
     (result_text, result_benignity) = self.pred.run_text(result)
     if not (quiet_mode and result_benignity):
-      print result_text,
+      sys.stdout.write("%s " % result_text)
+      sys.stdout.flush()
       self._print_name()
       sys.stdout.flush()
     if sandbox is not None and result != 1 and cleanup_mode:
@@ -1138,7 +1287,7 @@ def run_one_test(n, test_list, parallel = 0, finished_tests = None):
   """
 
   if (n < 1) or (n > len(test_list) - 1):
-    print "There is no test", `n` + ".\n"
+    print("There is no test %s.\n" % n)
     return 1
 
   # Run the test.
@@ -1167,7 +1316,7 @@ def _internal_run_tests(test_list, testnums, parallel):
       if run_one_test(testnum, test_list) == 1:
           exit_code = 1
   else:
-    number_queue = Queue.Queue()
+    number_queue = queue.Queue()
     for num in testnums:
       number_queue.put(num)
 
@@ -1185,7 +1334,7 @@ def _internal_run_tests(test_list, testnums, parallel):
     results.sort()
 
     # terminate the line of dots
-    print
+    print("")
 
     # all tests are finished, find out the result and print the logs.
     for (index, result, stdout_lines, stderr_lines) in results:
@@ -1204,35 +1353,39 @@ def _internal_run_tests(test_list, testnums, parallel):
 
 def usage():
   prog_name = os.path.basename(sys.argv[0])
-  print "%s [--url] [--fs-type] [--verbose|--quiet] [--parallel] \\" % \
-        prog_name
-  print "%s [--enable-sasl] [--cleanup] [--bin] [<test> ...]" \
-      % (" " * len(prog_name))
-  print "%s " % (" " * len(prog_name))
-  print "%s [--list] [<test> ...]\n" % prog_name
-  print "Arguments:"
-  print " test          The number of the test to run (multiple okay), " \
-        "or all tests\n"
-  print "Options:"
-  print " --list          Print test doc strings instead of running them"
-  print " --fs-type       Subversion file system type (fsfs or bdb)"
-  print " --http-library  DAV library to use (neon or serf)"
-  print " --url           Base url to the repos (e.g. svn://localhost)"
-  print " --verbose       Print binary command-lines (not with --quiet)"
-  print " --quiet         Print only unexpected results (not with --verbose)"
-  print " --cleanup       Whether to clean up"
-  print " --enable-sasl   Whether to enable SASL authentication"
-  print " --parallel      Run the tests in parallel"
-  print " --bin           Use the svn binaries installed in this path"
-  print " --use-jsvn      Use the jsvn (SVNKit based) binaries. Can be\n" \
-        "                 combined with --bin to point to a specific path"
-  print " --development   Test development mode: provides more detailed test\n"\
-        "                 output and ignores all exceptions in the \n"  \
-        "                 run_and_verify* functions. This option is only \n" \
-        "                 useful during test development!"
-  print " --server-minor-version  Set the minor version for the server.\n" \
-        "                 Supports version 4 or 5."
-  print " --help          This information"
+  print("%s [--url] [--fs-type] [--verbose|--quiet] [--parallel] \\" %
+        prog_name)
+  print("%s [--enable-sasl] [--cleanup] [--bin] [<test> ...]"
+      % (" " * len(prog_name)))
+  print("%s " % (" " * len(prog_name)))
+  print("%s [--list] [<test> ...]\n" % prog_name)
+  print("Arguments:")
+  print(" <test>  The number of the test to run, or a range of test\n"
+        "         numbers, like 10:12 or 10-12. Multiple numbers and\n"
+        "         ranges are ok. If you supply none, all tests are run.\n")
+  print("Options:")
+  print(" --list          Print test doc strings instead of running them")
+  print(" --fs-type       Subversion file system type (fsfs or bdb)")
+  print(" --http-library  DAV library to use (neon or serf)")
+  print(" --url           Base url to the repos (e.g. svn://localhost)")
+  print(" --verbose       Print binary command-lines (not with --quiet)")
+  print(" --quiet         Print only unexpected results (not with --verbose)")
+  print(" --cleanup       Whether to clean up")
+  print(" --enable-sasl   Whether to enable SASL authentication")
+  print(" --parallel      Run the tests in parallel")
+  print(" --bin           Use the svn binaries installed in this path")
+  print(" --use-jsvn      Use the jsvn (SVNKit based) binaries. Can be\n"
+        "                 combined with --bin to point to a specific path")
+  print(" --development   Test development mode: provides more detailed test\n"
+        "                 output and ignores all exceptions in the \n"
+        "                 run_and_verify* functions. This option is only \n"
+        "                 useful during test development!")
+  print(" --server-minor-version  Set the minor version for the server.\n"
+        "                 Supports version 4 or 5.")
+  print(" --fsfs-sharding Default shard size (for fsfs)\n"
+        " --fsfs-packing  Run 'svnadmin pack' automatically")
+  print(" --config-file   Configuration file for tests.")
+  print(" --help          This information")
 
 
 # Main func.  This is the "entry point" that all the test scripts call
@@ -1258,10 +1411,15 @@ def run_tests(test_list, serial_only = False):
   global svnadmin_binary
   global svnlook_binary
   global svnsync_binary
+  global svndumpfilter_binary
   global svnversion_binary
   global command_line_parsed
   global http_library
+  global fsfs_sharding
+  global fsfs_packing
+  global config_file
   global server_minor_version
+  global use_jsvn
 
   testnums = []
   # Should the tests be listed (as opposed to executed)?
@@ -1270,15 +1428,17 @@ def run_tests(test_list, serial_only = False):
   parallel = 0
   svn_bin = None
   use_jsvn = False
+  config_file = None
 
   try:
     opts, args = my_getopt(sys.argv[1:], 'vqhpc',
                            ['url=', 'fs-type=', 'verbose', 'quiet', 'cleanup',
                             'list', 'enable-sasl', 'help', 'parallel',
-                            'bin=', 'http-library=', 'server-minor-version=', 
-                            'use-jsvn', 'development'])
+                            'bin=', 'http-library=', 'server-minor-version=',
+                            'fsfs-packing', 'fsfs-sharding=',
+                            'use-jsvn', 'development', 'config-file='])
   except getopt.GetoptError, e:
-    print "ERROR: %s\n" % e
+    print("ERROR: %s\n" % e)
     usage()
     sys.exit(1)
 
@@ -1289,10 +1449,36 @@ def run_tests(test_list, serial_only = False):
     elif arg.startswith('BASE_URL='):
       test_area_url = arg[9:]
     else:
+      appended = False
       try:
         testnums.append(int(arg))
+        appended = True
       except ValueError:
-        print "ERROR:  invalid test number '%s'\n" % arg
+        # Do nothing for now.
+        appended = False
+
+      if not appended:
+        try:
+          # Check if the argument is a range
+          numberstrings = arg.split(':');
+          if len(numberstrings) != 2:
+            numberstrings = arg.split('-');
+            if len(numberstrings) != 2:
+              raise ValueError
+          left = int(numberstrings[0])
+          right = int(numberstrings[1])
+          if left > right:
+            raise ValueError
+
+          for nr in range(left,right+1):
+            testnums.append(nr)
+          else:
+            appended = True
+        except ValueError:
+          appended = False
+
+      if not appended:
+        print("ERROR: invalid test number or range '%s'\n" % arg)
         usage()
         sys.exit(1)
 
@@ -1334,10 +1520,15 @@ def run_tests(test_list, serial_only = False):
     elif opt == '--http-library':
       http_library = val
 
+    elif opt == '--fsfs-sharding':
+      fsfs_sharding = int(val)
+    elif opt == '--fsfs-packing':
+      fsfs_packing = 1
+
     elif opt == '--server-minor-version':
       server_minor_version = int(val)
       if server_minor_version < 4 or server_minor_version > 6:
-        print "ERROR: test harness only supports server minor version 4 or 5"
+        print("ERROR: test harness only supports server minor version 4 or 5")
         sys.exit(1)
 
     elif opt == '--use-jsvn':
@@ -1346,6 +1537,12 @@ def run_tests(test_list, serial_only = False):
     elif opt == '--development':
       setup_development_mode()
 
+    elif opt == '--config-file':
+      config_file = val
+
+  if fsfs_packing is not None and fsfs_sharding is None:
+    raise Exception('--fsfs-packing requires --fsfs-sharding')
+
   if test_area_url[-1:] == '/': # Normalize url to have no trailing slash
     test_area_url = test_area_url[:-1]
 
@@ -1354,25 +1551,24 @@ def run_tests(test_list, serial_only = False):
     sys.exit(1)
 
   # Calculate pristine_url from test_area_url.
-  pristine_url = test_area_url + '/' + pristine_dir
-  if windows:
-    pristine_url = pristine_url.replace('\\', '/')
+  pristine_url = test_area_url + '/' + pathname2url(pristine_dir)
 
   if use_jsvn:
-    if svn_bin is None: 
+    if svn_bin is None:
       svn_bin = ''
     svn_binary = os.path.join(svn_bin, 'jsvn' + _bat)
     svnadmin_binary = os.path.join(svn_bin, 'jsvnadmin' + _bat)
     svnlook_binary = os.path.join(svn_bin, 'jsvnlook' + _bat)
     svnsync_binary = os.path.join(svn_bin, 'jsvnsync' + _bat)
+    svndumpfilter_binary = os.path.join(svn_bin, 'jsvndumpfilter' + _bat)
     svnversion_binary = os.path.join(svn_bin, 'jsvnversion' + _bat)
-    use_jsvn = False
   else:
     if svn_bin:
       svn_binary = os.path.join(svn_bin, 'svn' + _exe)
       svnadmin_binary = os.path.join(svn_bin, 'svnadmin' + _exe)
       svnlook_binary = os.path.join(svn_bin, 'svnlook' + _exe)
       svnsync_binary = os.path.join(svn_bin, 'svnsync' + _exe)
+      svndumpfilter_binary = os.path.join(svn_bin, 'svndumpfilter' + _exe)
       svnversion_binary = os.path.join(svn_bin, 'svnversion' + _exe)
 
   command_line_parsed = True
@@ -1386,11 +1582,11 @@ def run_tests(test_list, serial_only = False):
 
   if not testnums:
     # If no test numbers were listed explicitly, include all of them:
-    testnums = range(1, len(test_list))
+    testnums = list(range(1, len(test_list)))
 
   if list_tests:
-    print "Test #  Mode   Test Description"
-    print "------  -----  ----------------"
+    print("Test #  Mode   Test Description")
+    print("------  -----  ----------------")
     for testnum in testnums:
       TestRunner(test_list[testnum], testnum).list()
 
@@ -1402,12 +1598,12 @@ def run_tests(test_list, serial_only = False):
   if serial_only or len(testnums) < 2:
     parallel = 0
 
-  # Setup the pristine repository
-  actions.setup_pristine_repository()
-
   # Build out the default configuration directory
   create_config_dir(default_config_dir)
 
+  # Setup the pristine repository
+  actions.setup_pristine_repository()
+
   # Run the tests.
   exit_code = _internal_run_tests(test_list, testnums, parallel)
 
